#!/usr/bin/env node
const fs = require('fs');
const ArgumentParser = require('argparse').ArgumentParser;
const parse = require('lcov-parse');
const badge = require('badge-up');
const svg2png = require('svg2png');
const pkg = require('./package.json');

const defaultOutput = './badge.svg';
const defaultLabel = 'coverage';

const parser = new ArgumentParser({
  version: pkg.version
});
parser.addArgument(
  'input'
);
parser.addArgument(
  [ '-o', '--output' ],
  {
    defaultValue: defaultOutput
  }
);
parser.addArgument(
  [ '-l', '--label' ],
  {
    defaultValue: defaultLabel
  }
);
const args = parser.parseArgs();

const format = args.output.slice(-3);
if (format !== 'svg' && format !== 'png') {
  parser.printHelp();
  return;
}

parse(args.input, (parseErr, data) => {
  if (parseErr) {
    throw parseErr;
  }

  const summary = {
    lines: { found: 0, hit: 0 },
    branches: { found: 0, hit: 0 },
    functions: { found: 0, hit: 0 },
  };
  const keys = ['lines', 'branches', 'functions'];

  data.forEach((arg) => {
    keys.forEach((key) => {
      summary[key].found += arg[key].found;
      summary[key].hit += arg[key].hit;
    });
  });

  const average = Math.round(keys.reduce((avg, key) => {
    const found = summary[key].found;
    return avg + (found > 0
      ? summary[key].hit / found * 100
      : 100);
  }, 0) / keys.length * 100) / 100;

  let color = '';
  if (average <= 50) {
    color = 'red';
  } else if (average <= 60) {
    color = 'orange';
  } else if (average <= 70) {
    color = 'yellow';
  } else if (average <= 80) {
    color = 'yellowgreen';
  } else if (average <= 90) {
    color = 'green';
  } else {
    color = 'brightgreen';
  }

  badge(args.label, `${average}%`, badge.colors[color], (badgeErr, output) => {
    if (badgeErr) {
      throw badgeErr;
    }

    if (format === 'png') {
      output = svg2png.sync(output);
    }

    fs.writeFile(args.output, output, (writeErr) => {
      if (writeErr) {
        throw writeErr;
      }
    });
  });
});
